import pathlib
import os, getpass, shutil, re, psutil, string, random
import pandas
from collections import namedtuple
# import sh
import logging
# import filecmp
import datetime
# import numpy
# import itertools
import subprocess
import json
from packaging import version
from bohra.CustomLog import CustomFormatter


# Logger
LOGGER =logging.getLogger(__name__) 
LOGGER.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
ch.setFormatter(CustomFormatter())
fh = logging.FileHandler('bohra.log')
fh.setLevel(logging.DEBUG)
formatter = logging.Formatter('[%(levelname)s:%(asctime)s] %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p') 
fh.setFormatter(formatter)
LOGGER.addHandler(ch) 
LOGGER.addHandler(fh)

class InitBohra(object):

    def __init__(self):

        self.script_path = f"{pathlib.Path(__file__).parent}"
    
    def init(self):

        LOGGER.info(f"Will now try to install dependencies. Please be patient this may take some time!!... Maybe get coffee.")
        process = subprocess.Popen(['bash', f"{self.script_path}/bohra_install.sh"], stdout=subprocess.PIPE, encoding='utf-8')
        while process.poll() is None:
            l = process.stdout.readline().strip() # This blocks until it receives a newline.
            print(f"{l}")
        


class RunSnpDetection(object):
    '''
    A class for Bohra pipeline object
    '''
    
    def __init__(self, args):
        
        # user
        self.user = getpass.getuser()
        # get date and time
        self.now = datetime.datetime.today().strftime("%d_%m_%y_%H")
        self.day = datetime.datetime.today().strftime('%Y-%m-%d')
        self.script_path = f"{pathlib.Path(__file__).parent}"
        # get the working directory
        
        self.assembler = args.assembler
        self.kraken_db = args.kraken_db
        self.blast_db = args.blast_db
        self.data_dir = args.data_dir
        self.mobsuite_db = args.mobsuite_db if args.mobsuite_db != "" else "no_db"
        self.workdir = pathlib.Path(args.workdir)        
        LOGGER.info(f"\033[1mBohra is being run in {self.workdir} by {self.user} on {self.day}.\033[0m")
        
        # path to pipeline resources
        self.pipeline = args.pipeline
        self.preview = True if self.pipeline == 'preview' else False
        LOGGER.info(f"You are running bohra in {self.pipeline} mode.")
        self.job_id =args.job_id
        LOGGER.info(f"Job ID is set {self.job_id}")
        # path to reference and mask
        self.ref = args.reference
        # LOGGER.info(f"The reference is {self.ref}")
        # self.link_file(self.ref)
        self.mask = args.mask
        # path to input file
        self.reads,self.contigs = self._check_input_files(_input = args.input_file, contigs = args.contigs, pipeline = self.pipeline)

        self.keep = True if args.keep == 'Y' else False
        # snippy args
        self.minmap = args.minmap
        self.mincov = args.mincov
        self.basequal = args.basequal
        self.minqual = args.minqual
        self.minfrac = args.minfrac
        
        self.gubbins = args.gubbins
        self.force = args.force        
        self.cpus = args.cpus
        self.config = args.config
        self.profile_config = os.getenv('BOHRA_PROFILES','')
        self.profile = self._get_profile() if args.profile == '' else args.profile
        self.run_kraken = 'false'
        self.no_phylo = args.no_phylo
        self.abritamr_args = args.abritamr_args
        self.spades_args = args.spades_args
        self.proceed = args.proceed
        self.use_conda = True if args.no_conda == False else False
        self.conda_path = args.conda_path
        
        
    def _check_input_files(self, _input, contigs, pipeline):
        
        if _input == '':
            LOGGER.critical(f"You must supply and input file. Please see help and try again.")
            raise SystemExit

        # if _input == '':
        #     if pipeline in ['preview', 'snps','phylogeny','default','full']:
        #         LOGGER.critical(f"You are trying to run the {pipeline} bohra pipeline - you must supply an input file with paths to reads.")
        #         raise SystemExit
        
        return pathlib.Path(_input), contigs


    def _get_profile(self):

        # get hostname
        LOGGER.info(f"Tyring to find your profile.")
        profile = 'lcl'
        if self.profile_config != '' and self._path_exists(pathlib.Path(self.profile_config)):
            with open(self.profile_config, 'r') as j:
                _cfg = json.load(j)

            p = subprocess.run('hostname', shell = True, capture_output = True, encoding = "utf-8")
            host = p.stdout.strip()
            LOGGER.info(f"Host is : {host}")
            if host in _cfg:
                profile = _cfg[host]
        if profile == 'no_config':
            LOGGER.critical(f"It seems you on an MDU system and trying to run on a head node. Please move to a compute node and try again.")
            raise SystemExit
        LOGGER.info(f"You are running bohra with the {profile} profile.")
        return profile
            
    def _set_cpu_limit_local(self, cpus):

        total_cores = os.cpu_count()
        one,five,fifteen = psutil.getloadavg()
        avail = total_cores - max(one,five,fifteen)

        if int(cpus) == 0:
            return avail
        elif int(cpus) < avail:
            return cpus
        else:
            return avail

    def _check_config(self, config):

        if self._path_exists(pathlib.Path(config)):
            with open(config, 'r') as c:
                data = c.read()
                if self.profile in data:
                    LOGGER.info(f"Profile : {self.profile}.")
                else:
                    LOGGER.warning(f"Profile : {self.profile} is not found in {config}. I hope you know what you are doing!")
                return config
        else:
            LOGGER.critical(f"You have provided the path to an alternative config file, which does not exist. Please try again.")
            raise SystemExit

    def _generate_random_string(self):

        letters = string.ascii_lowercase
        result_str = ''.join(random.choice(letters) for i in range(10))

        return result_str
    
    def setup_for_rerun(self):
        
        report_path_orig = self.workdir / 'report'
        report_path_preview =  self.workdir / f'report_archived_{self.day}_{self._generate_random_string()}'
        if self.keep and report_path_orig.exists():
            cmd = f"mv {report_path_orig} {report_path_preview}"
            LOGGER.info(f"Archiving previous report directory...")
            self._run_subprocess(cmd  = cmd)
        


    def _path_exists(self,path, v = True):
        '''
        ensure files are present, if so continues if not quits with FileNotFoundError
        input:
            :path: patht to files for pipeline
            :v: if v == True print message, else just check
        output:
            returns True (or fails with FileNotFoundError)
        '''
        
        if path.exists() and os.access(path, os.R_OK):
            if v == True:
                LOGGER.info(f"Found {path.name}.")
            return True
        else:
            # LOGGER.warning(f"The {path.name} does not exist or is not accessible.")
            return False

    
    def _name_exists(self, name):
        '''
        check if the name is an empty string JOB id can not be empty
       
        '''
        
        if isinstance(name, str):
            if len(name) == 0:
                LOGGER.warning('Job id ca not be empty, please set -j job_id to try again')
                raise SystemExit()
            else:
                return name
        else:
            LOGGER.warning('Job id ca not be empty, please set -j job_id to try again')
            raise SystemExit()
   

    def _check_ref(self, ref, pipeline):
        
        if pipeline in ['snps','default','full','phylogeny']:
            if self.ref == '':
                LOGGER.critical(f"Reference file must be provided. Please try again.")
                raise SystemExit
            elif not self._path_exists(pathlib.Path(self.ref), v = True):
                LOGGER.critical(f"A valid reference file must be provided. Please try again.")
                raise SystemExit
            
            LOGGER.info(f"Reference {self.ref} has been found. Will now copy to running directory.")
            reference = self._copy_files(_file = self.ref)
            LOGGER.info(f"Checking if reference is a valid reference file.")
            p = subprocess.run(f"any2fasta {ref}", shell = True, capture_output = True, encoding = "utf-8")
            if p.returncode == 0:
                LOGGER.info(f"Reference is in a valid format.")
            
            else:
                LOGGER.critical(f"There is something wrong with your reference file. Valid file types are .fasta, .gbk, .fasta.gz, .gbk.gz. Please check your inputs and try again.")
                raise SystemExit
            return reference
        else:
            return 'no_ref_req'
        
    def _check_gzip(self,read):
        """
        ensure that gz file is true gz
        input:
            read: path to read file
        """
        LOGGER.info(f"Checking if file is valid gzip.")
        if f"{read}".endswith('gz'):
            p = subprocess.run(f"gzip --test {read}", shell= True, capture_output= True, encoding='utf-8')
            if p.returncode != 0:
                LOGGER.critical(f"Something is wrong with {read}. It is not a valid gzipped file. Please check your inputs and try again.")
                raise SystemExit
            return True
        else:
            True
   
    
    def _check_size_file(self, path):
        '''
        check the size of a file
        '''
        s = path.stat().st_size
        return s

    def _check_kraken2_files(self, k2db):
        '''
        ensure that kraken2 DB is not empty
        '''
        LOGGER.info(f'Checking that {k2db} is a directory, checking that files are not empty')
        if pathlib.Path(k2db).is_dir():
                LOGGER.info(f'Found {k2db}, checking that files are not empty')
                kmerfiles = sorted(pathlib.Path(k2db).glob('*'))
                s = []
                for k in range(len(kmerfiles)):
                    s.append(self._check_size_file(pathlib.Path(k2db) / kmerfiles[k]))
                if 0 not in s:
                    self.run_kraken = 'true'
                    return True
        else:
            LOGGER.critical(f'{k2db} is not a directory.')
            return False
        

    def _check_kraken2DB(self, checking = False):
        '''
        ensure that DB is present and not emtpy
        '''
        
        LOGGER.info(f'Searching for kraken2 DB:  $KRAKEN2_DEFAULT_DB')
        if os.getenv('KRAKEN2_DEFAULT_DB'):
            try: # check that there is an environment value set for kraken2 db
                k2db = pathlib.Path(os.environ["KRAKEN2_DEFAULT_DB"])
                LOGGER.info(f"You are using the default kraken2 database at : {os.environ['KRAKEN2_DEFAULT_DB']}")
                if self._check_kraken2_files(k2db = k2db):
                    self.kraken_db = f"{k2db}"
                    # LOGGER.info(f"You are using the deafult kraken2 database at : {os.environ['KRAKEN2_DEFAULT_DB']}")
                LOGGER.info(f"Congratulations your kraken database is present and all files are present.")

            except KeyError:
                LOGGER.critical(f"It seems that your settings for the kraken DB are incorrect, speciation will not be performed.")
                self.run_kraken = False # don't run kraken
        
        else:
            
            if not checking:
                if pathlib.Path(self.kraken_db).exists():
                    LOGGER.info(f"{self.kraken_db} has been found.")
                    self.check_kraken2_files(k2db = self.kraken_db)
                else:
                    LOGGER.critical(f"It seems that your settings for the kraken DB are incorrect, speciation will not be performed.")           
                    self.run_kraken = False
            else:
                LOGGER.warning('You do not have a default kraken2 DB. You have 4 options \n1). bohra run --kraken_db to point to a specific directory \n2). do not perform speciation \n3). set a KRAKEN2_DEFAULT_DB environment variable or 4). use bohra krakendb_download.')
        
    def _check_mobsuite_db(self):
        
        if self.mobsuite_db != "no_db":
            
            if pathlib.Path(self.mobsuite_db).exists() and os.access(self.mobsuite_db, os.W_OK) :
                LOGGER.info(f"Your mobsuite database is set up and should be ready to go.")
            else:
                LOGGER.critical(f"Your mobsuite database is not configured properly. Please check your setup and try again.")
                raise SystemExit
        else:
            LOGGER.info(f"You are using the default mobsuite database in your enviornment. May the force be with you.")

    def _get_db(self, _type):

        p = subprocess.run(f"mlst -h", shell = True, capture_output = True, encoding = "utf-8")
        db = [d for d in p.stdout.strip().split('\n') if _type in d]
        db = db[0].split('\'')[-2].strip('\'')
        if db != '':
            LOGGER.info(f"Found : {db}")
            return db
        else:

            LOGGER.critical(f"There seems to be something wrong with your mlst setup. Please check and try again.")
            raise SystemExit

    def _check_mlstdb(self):

        LOGGER.info(f"Checking mlst setup.")
        if self.blast_db == "" or self.data_dir == "":
            LOGGER.warning(f"You do not have mlst databases pre-configured the default DB with your installation of mlst will be used.")
            # self.blast_db = self._get_db('--blastdb')
            # self.data_dir = self._get_db('--datadir')
        elif self._path_exists(pathlib.Path(self.blast_db)) and self._path_exists(pathlib.Path(self.data_dir)):
            LOGGER.info(f"Your mlst databases have been found.")
        else:
            LOGGER.critical(f"There seems to be something wrong with your mlst setup. Please check and try again.")
            raise SystemExit
        
        return True


    def _check_installation(self,software):
        '''
        Check that software is installed
        input:
            :software: the name of the software - must be command line name 
        '''

        if shutil.which(software):
            LOGGER.info(f"{software} detected.")
        else:
            LOGGER.critical(f"{software} is not installed, please check dependencies and try again.")
            raise SystemExit
    
    def check_dependencies(self, checking):

                
        software = {
            'nextflow':'nextflow -v',
            'snippy': 'snippy --version 2>&1',
            'snp-dists': 'snp-dists -v 2>&1',
            'iqtree': 'iqtree --version 2>&1',
            'kraken2': 'kraken2 -v 2>&1',
            'seqkit': 'seqkit version 2>&1',
            'any2fasta': 'any2fasta -v',
            'gubbins': 'run_gubbins.py --version 2>&1',
            'csvtk':'csvtk version 2>&1',
            'panaroo': 'panaroo --version 2>&1',
            'shovill': 'shovill -v 2>&1',
            'skesa': 'skesa -v 2>&1',
            'spades': 'spades.py -v 2>&1',
            'abritamr': 'abritamr -v 2>&1',
            'mlst': 'mlst -v 2>&1',
            'prokka': 'prokka -v 2>&1',
            'mash': 'mash --version 2>&1',
            'quicktree': 'quicktree -v 2>&1',
            'amrfinderplus': 'amrfinder --version 2>&1',
            'abritamr': 'abritamr -v 2>&1',
            'mob_recon': 'mob_recon -V 2>&1'
        }

        software_versions = [f"Software"] # a list of versions for output
        LOGGER.info(f"Checking that dependencies are installed and recording version.")
        version_pat_3 = re.compile(r'\bv?(?P<major>[0-9]+)\.(?P<minor>[0-9]+)(?:\.(?P<release>[0-9]+)*)?(?:\.(?P<build>[0-9]+)*)?\b')
        
        for sft in software:
            p = subprocess.run(software[sft], capture_output=True, encoding = "utf-8", shell = True)
            p = p.stdout
            v = version_pat_3.search(p.strip())
            if v:
                v = v.group(0)
                software_versions.append(f"{sft} {v}")
                LOGGER.info(f"{sft} version {v} detected.")              
            else:
                LOGGER.critical(f"{sft} is not installed.")
                raise SystemExit
        LOGGER.info(f"Now checking kraken2 DB")
        self._check_kraken2DB(checking = checking)
        LOGGER.info(f"Checking mobsuite database setup.")
        self._check_mobsuite_db()
        software_versions.append(f"kraken2 DB: {self.kraken_db}")
        if not checking:
            # print(checking)
            self._check_mlstdb()
            software_versions.append(f"mlst blast db: {self.blast_db}")
            software_versions.append(f"mlst data dir: {self.data_dir}")
        # write software_versions.tab
        
            LOGGER.info(f"Writing software_versions.txt")
            if not pathlib.Path('report').exists():
                subprocess.run(f"mkdir -p report", shell = True)
            pathlib.Path('report','software_versions.txt').write_text('\n'.join(software_versions))
        LOGGER.info(f"\033[1mCongratulations all dependencies are installed.\033[0m")
        return True 

    def _check_shape(self, val, reads = True):
        if reads and val == 3:
            return True
        elif val == 2:
            return True
        else:
            return False
    
    def _check_gzip(self,read):
        """
        ensure that gz file is true gz
        input:
            read: path to read file
        """
        LOGGER.info(f"Checking if file is valid gzip.")
        if f"{read}".endswith('gz'):
            p = subprocess.run(f"gzip --test {read}", shell= True, capture_output= True, encoding='utf-8')
            if p.returncode != 0:
                LOGGER.critical(f"Something is wrong with {read}. It is not a valid gzipped file. Please check your inputs and try again.")
                raise SystemExit
            return True
        else:
            True
   
    def _check_reads(self,reads):

        if reads != '' and self._path_exists(reads):
            
            tab = pandas.read_csv(reads, sep = '\t', header = None, dtype = str)
            if self._check_shape(tab.shape[1]):
                LOGGER.info(f"File {reads} is in correct format.")
            else:
                LOGGER.critical(f"{reads} is not in the correct format. Please check your inputs and try again.")
                raise SystemExit
            return tab
        # elif self.pipeline in ['assemble','amr_typing']:
        #     return pandas.DataFrame()
        else:
            LOGGER.critical(f"Something is wrong with {reads}. Please try again.")
            raise SystemExit


    def _check_contigs(self, contigs):

        if contigs != '' and self._path_exists(pathlib.Path(contigs)):
            tab = pandas.read_csv(contigs, sep = '\t', header = None, dtype = str)
            if self._check_shape(tab.shape[1], reads = False):
                LOGGER.info(f"File {contigs} is in correct format.")
                return True
            else:
                LOGGER.critical(f"{contigs} is not in the correct format. Assemblies will be generated")
                return False
        else:
            LOGGER.info(f"No valid contigs file has been supplied. Assemblies will be generated.")
            return False

    def _check_phylo(self, isolates_list):
        LOGGER.info(f"Checking if phylo needs to be run.")
        LOGGER.info(f"Your analysis contains {len(isolates_list)}")
        if self.pipeline == 'preivew' and len(isolates_list) > 2:
            LOGGER.info(f"You are running in preview mode, a mash tree will be output")
        if self.pipeline in ['amr_typing', 'assemble']:
            LOGGER.info(f"Your are running the {self.pipeline} bohra pipeline. No tree will be generated.")
            self.no_phylo = True
        elif len(isolates_list) < 3 and self.pipeline in ['phylogeny', 'default', 'all']:
            LOGGER.warning(f"You have less than 3 isolates in your dataset, no phylogenetic tree will be generated.")
            self.no_phylo = True
        elif not self.no_phylo:
            LOGGER.info(f"A phylogenetic tree will be generated.")
        elif self.pipeline == 'all' and len(isolates_list) < 4:
            LOGGER.warning(f"You can not run panaroo with less than 4 isolates.")
            self.pipeline = 'default'
        
        # if self.no_phylo:
        #     LOGGER.info(f"You have selected no_phylo option so no phylogenetic tree will be generated.")
        
    def _copy_files(self, _file):

        p = pathlib.Path(_file)
        name = p.name
        if pathlib.Path(self.workdir, name).exists():
            LOGGER.info(f"The file : {name} already exists in the current directory")
        else:
            cmd = f"cp {p} {pathlib.Path(self.workdir, name)}"
            LOGGER.info(f"Running : {cmd}")
            subprocess.run(cmd, shell = True)

        return f"{name}"
    
    def _link_input_files(self, iso_dir,read, target):

        if read.exists() and not pathlib.Path(f"{iso_dir}/{target}").exists():
                    subprocess.run(f"ln -sf {read} {iso_dir}/{target}", shell = True)
        elif not read.exists():
            LOGGER.critical(f"{read} is not a valid path. All read files must be valid path. Please try again.")
            raise SystemExit
    
    def _check_reads_not_equal(self, r1,r2):

        if f"{r1}" == f"{r2}":
            LOGGER.critical(f"It seems that {r1} and {r2} are the same name. You must supply paths to R1 and R2. Please check your inputs and try again.")
            raise SystemExit

    def _setup_directory(self, reads):
        
        isolates_list = []
        if not reads.empty:
            LOGGER.info(f"Setting up isolate directories.")
            for row in reads.iterrows():
                if not row[1][0].startswith('#'):
                    isolates_list.append(row[1][0])
                    iso_dir = self.workdir/ f"{row[1][0]}" 
                    if not iso_dir.exists():
                        subprocess.run(f"mkdir {iso_dir}", shell = True)
                    # for r in [row[1][1],row[1][2]]:
                    read1 = pathlib.Path(row[1][1])
                    read2 =pathlib.Path(row[1][2])
                    self._check_reads_not_equal(r1 = read1, r2 = read2)
                    target1 = 'R1.fastq.gz'
                    target2 = 'R2.fastq.gz'
                    self._link_input_files(iso_dir = iso_dir, read = read1, target = target1)
                    self._link_input_files(iso_dir = iso_dir, read = read2, target = target2)       
            # self._check_phylo(isolates_list = isolates_list)
        
        elif self.contigs != '' and pathlib.Path(self.contigs).exists():
            tab = pandas.read_csv(self.contigs, sep = '\t', header = None, names = ['Isolate','Path'])
            for row in tab.iterrows():
                if not row[1][0].startswith('#'):
                    isolates_list.append(row[1][0])
                    iso_dir = self.workdir/ f"{row[1][0]}" 
                    if not iso_dir.exists():
                        subprocess.run(f"mkdir {iso_dir}", shell = True)
                    # for r in [row[1][1],row[1][2]]:
                    contig = pathlib.Path(row[1][1])
                    target = 'contigs.fa'
                    self._link_input_files(iso_dir = iso_dir, read = contig, target = target)
        
        else:
            LOGGER.critical(f"There seems to be a problem with your input files... not isolates can be extracted. Please check you inputs and try again.")
            raise SystemExit
        self._check_phylo(isolates_list = isolates_list)
        LOGGER.info(f"Updating isolate list.")
        pathlib.Path(f"isolates.list").write_text('\n'.join(isolates_list))
        return f"isolates.list"
        
    def _check_snippy_defaults(self, snippy_arg, val):
        
        _file = pathlib.Path(self.script_path, 'nextflow.config')

        if _file.exists():
            with open(f"{_file}", 'r') as fh:
                lines = fh.readlines()
                for line in lines:
                    if snippy_arg in line:
                        default_val = line.split('=')[-1].strip().strip("\"")
                        if f"{default_val}" == f"{val}":
                            # LOGGER.info(f"You are using default setting for {snippy_arg}")
                            return False
                        else:
                            LOGGER.info(f"You have elected to set your own value for {snippy_arg}. Good luck, we hope you know what you are doing ;)")
                            return True

    def _generate_snippy_params(self):

        _dict = {
                'mapqual': self.minmap,
                'basequal': self.basequal,
                'mincov':self.mincov,
                'minfrac':self.minfrac,
                'minqual':self.minqual
                }
        snippy_opts = []

        for d in _dict:
            if self._check_snippy_defaults(snippy_arg=d, val = _dict[d]):
                snippy_opts.append(f"--{d} {_dict[d]}")
        
        return ' '.join(snippy_opts)
    
    def _generate_spades_args(self):

        if self.assembler == 'spades':
            return f"--spades_opt '{self.spades_args}'"
        else:
            return ''

    def _generate_cmd(self, mode, run_kraken, kraken2_db,assembler, mask_string, reference, 
                        isolates, user, day, contigs, run_iqtree, species, cpus, config, profile, gubbins,blast_db,data_dir, job_id):
        
        stub = f"nextflow -Dnxf.pool.type=sync run {self.script_path}/main.nf"
        resume = '' if self.force else "-resume"
        cpu = f'-executor.cpus={int(cpus)}' if cpus != '' else ''
        config = f'-c {config}' if config != '' else ''
        conda = '--enable_conda true' if self.use_conda else ''
        conda_path = f"--conda_path {self.conda_path}" if self.conda_path != '' else ''
        snippy_opts = self._generate_snippy_params()
        spades_opts = self._generate_spades_args()

        parameters = f"--job_id {job_id} --mode {mode} --run_iqtree {run_iqtree} --run_kraken {run_kraken} --kraken2_db {kraken2_db} --assembler {assembler} --mask_string {mask_string} --reference {reference} --contigs_file {contigs} --species {species if species != '' else 'no_species'} --outdir {self.workdir} --isolates {isolates} --user {user} --day {day} --gubbins {gubbins} --blast_db {blast_db} --data_dir {data_dir} --mobsuite_db {self.mobsuite_db} {conda} {conda_path} {snippy_opts} {spades_opts}"
        options = f"-with-report bohra_{day}_report.html -with-trace -profile {profile} {resume} {cpu} {config} {'-with-conda' if self.use_conda else ''}"

        cmd = f"{stub} {parameters} {options}"
        return cmd
        
    def _run_subprocess(self, cmd):

        LOGGER.info(f"Running:\n\033[1m{cmd}\033[0m")
        p = subprocess.run(cmd, shell = True)
        return p
    
    def _run_cmd(self, cmd):
        
        if self.proceed:
            return self._run_subprocess(cmd= cmd)
        else:

            try:
                LOGGER.info(f"Please paste the following command to run the pipeline:\n\033[1m{cmd}\033[0m")
            except NameError:
                LOGGER.critical(f"It seems something has gone wrong with your inputs. \
Please select a mode to run, choices are 'analysis' or 'finish'")
                raise SystemExit

    def _write_details(self,cmd, reference, mask):

        col1 = ['Reference file', 'Mask file', 'Working directory', 'User', 'Date', 'Pipeline', 'Nextflow command']
        col2 = [reference, mask,self.workdir,self.user,self.day,self.pipeline, cmd]
        df = pandas.DataFrame({'detail':col1,'description':col2})
        LOGGER.info(f"Saving pipeline details")
        subprocess.run(f"mkdir -p {self.workdir}/report", shell = True)
        df.to_csv(f"{self.workdir}/report/details.txt", index =False, sep = '\t')

    def run_pipeline(self):
        '''
        run pipeline, if workflow runs to completion print out a thank you message.
        '''
        self.setup_for_rerun()
        # run checks for inputs
        if self.use_conda == False:
            LOGGER.warning(f"You are using a pre-configured conda environment - please note the results may be unexpected.")
            self.check_dependencies(checking=False)
        else:
            LOGGER.info(f"You are running with conda - wise decision!! Will now ensure that kraken DB is configured properly.")
            self._check_kraken2DB(checking = False)
            LOGGER.info(f"Checking mobsuite database setup.")
            self._check_mobsuite_db()
            LOGGER.info(f"Now looking for MLST setup")
            self._check_mlstdb()
        # input files
        reads = self._check_reads(reads = self.reads)
        contigs = self._check_contigs(contigs = self.contigs)
        
        # reference
        
        reference = self._check_ref(ref = self.ref,pipeline=self.pipeline)
        # mask
        if self.mask != '' and not self._path_exists(pathlib.Path(self.mask)):
            LOGGER.critical(f"{self.mask} is not a valid path please try again.")
            raise SystemExit
        elif self.mask != '' and  self._path_exists(pathlib.Path(self.mask, v = False)):
            self.mask = self._copy_files(_file = self.mask)
            LOGGER.info(f"Mask file {self.mask} has been provided.")
        else:
            LOGGER.info(f"No mask file has been provided.")

        isolates_list = self._setup_directory(reads = reads) 

        if contigs:
            contigs_file = self.contigs
        else:
            contigs_file = 'no_contigs'
            
        run_iqtree = 'false' if self.no_phylo else 'true'
        mask_string = 'no_mask' if self.mask == '' else self.mask
        if self.config == '':
            cpus = self._set_cpu_limit_local(self.cpus)
            config = ''
        else:
            config = self._check_config(self.config)
            cpus = ''
        
         
        cmd = self._generate_cmd(mode = self.pipeline, run_kraken = self.run_kraken, kraken2_db = self.kraken_db,
                        contigs = contigs_file, cpus = cpus, config = config, assembler = self.assembler, 
                        mask_string = mask_string, reference = reference, run_iqtree = run_iqtree,profile = self.profile,
                        isolates = isolates_list, day = self.day, user = self.user, 
                        species = self.abritamr_args, gubbins = self.gubbins, 
                        blast_db = self.blast_db if self.blast_db != '' else 'no_db', 
                        data_dir = self.data_dir if self.data_dir != '' else 'no_db', job_id = self.job_id)
        self._write_details(cmd = cmd, reference = reference, mask = mask_string)
        self._run_cmd(cmd)


class SetupInputFiles(RunSnpDetection):

    def __init__(self, args):

        self.read_path = args.read_path
        self.isolate_list = args.isolate_list
        self.now = datetime.datetime.today().strftime("%d_%m_%y_%H")
        self.day = datetime.datetime.today().strftime('%Y-%m-%d')

    def _check_path(self,path):

        """
        Check if path provided exists
        :input - path to where reads/contigs/inputfile are
        :output - boolean 
        """
        LOGGER.info(f"Checking for {path}")
        if pathlib.Path(path).exists():
            LOGGER.info(f"{path} exists.")
            return True
        else:
            LOGGER.critical(f"Path provided : {path} does not exist - please try again.")
            return False
        
    def _extract_isolates(self, isolate_file):
        
        with open(isolate_file, 'r') as f:
            isolates = f.read().strip().split('\n')
        
        return isolates
    
    def _get_isolate_list(self, _dir,  ext= f"*.f*q.gz"):

        all_data = sorted(_dir.rglob(ext))

        iso_found = set()

        for reads in all_data:
            nme = reads.name.split('_')[0]

            iso_found.add(nme)
        
        LOGGER.info(f"Found {len(list(iso_found))} distinct sample names at path {_dir}")
        return list(iso_found)
    
    

    def _glob_reads(self, _dir, isolates):

        iso_found = self._get_isolate_list(_dir = _dir, ext = "*.f*q.gz")
        
        isolist = []
        if isolates != []:

            for i in list(iso_found):
                for j in isolates:
                    if i in j:
                        isolist.append(i)
        else:
            isolist = iso_found

        lines = []

        for iso in isolist:

            reads = sorted(_dir.rglob(f"*{iso}*.f*q.gz"))
            if len(reads) == 2:
                LOGGER.info(f"Now add reads for {iso}")
                lines.append(f"{iso}\t{reads[0]}\t{reads[1]}")
            elif len(reads) <2:
                LOGGER.warning(f"There do not appear to be 2 reads for {iso}. Skipping")
            else:
                LOGGER.warning(f"There appear to be more than 2 reads available for {iso}. Skipping")
        if lines != []:
            LOGGER.info(f"Saving reads file as isolates.tab")
            pathlib.Path('isolates.tab').write_text('\n'.join(lines))
        else:
            LOGGER.warning(f"It appears that no reads have been found. Please check your input and try again.")
        
        return True

        
    def _glob_data(self, path, isolates =[], data_type = 'reads', ext = "f*q.gz"):

        """
        glob path provided for data
        :input - path provided.
                 data type (reads or contigs - default to reads)
                 ext extension of data type fastq is default
        :output - list of data
        """

        _dir = pathlib.Path(path).resolve()

        if data_type == 'reads':

            self._glob_reads(_dir = _dir, isolates= isolates)
        
    
    def find_reads(self):

        if self.read_path != '' and self._check_path(self.read_path):
            if self.isolate_list != '': 
                isolates = self._extract_isolates(isolate_file= self.isolate_list) if self._check_path(self.isolate_list) else []
            else:
                isolates = []
            self._glob_data(path = self.read_path, isolates= isolates)
        else:
            LOGGER.critical(f"There seems to be a problem with your read path. Please provide a valid path to the reads you wish to include")
            raise SystemExit

class TestBohra(SetupInputFiles):

    def __init__(self,args):

        self.isolate_list = ['ERR1102348','ERR1102353','ERR1102355','ERR1102356']
        self.download_stub = "https://raw.githubusercontent.com/MDU-PHL/bohra/master/data"
        self.reference = self._check_reference_test(args.reference)
        self.read_path = f"{pathlib.Path.cwd() / 'test_data'}"

    def _download_reads_from_github(self):

        for isolate in self.isolate_list:
            
            for r in [1,2]:
                cmd = f"mkdir -p test_data/{isolate} && wget -O test_data/{isolate}/{isolate}_{r}.fastq.gz {self.download_stub}/{isolate}/{isolate}_{r}.fastq.gz"
                self._run_subprocess(cmd = cmd)
            

    def _download_reference_from_github(self):

        cmd = f"wget -O Lm_Cluster1_J1-108.fa {self.download_stub}/Lm_Cluster1_J1-108.fa"
        self._run_subprocess(cmd = cmd)
        return 'Lm_Cluster1_J1-108.fa'
    
    def _check_reference_test(self, path):

        if not pathlib.Path(path).exists():

            return self._download_reference_from_github()

        else:
            return path

    def _check_test_data(self,path):

        for i in self.isolate_list:
            reads = sorted(pathlib.Path(path).glob(f"{i}/{i}*.f*q.gz"))
            if len(reads) != 2:
                LOGGER.warning(f"Reads for {i} are not found. Will try to get them for you!")
                return False
        LOGGER.info(f"All reads are found at {path}")
        return True
    
    def _make_test_input(self, path):

        LOGGER.info(f"Now generating input file for bohra.")
        self._glob_data(path = path)

    def run_tests(self):
        
        LOGGER.info(f"Checking availability of data.")
        if not pathlib.Path(self.read_path).exists():
            LOGGER.info(f"Will now download some reads for testing - this may take a little while - it might be coffee time.")
            self._download_reads_from_github()
            self._make_test_input(path = self.read_path)
        elif self._check_test_data(path = self.read_path):
            self._make_test_input(path = self.read_path)
        else:
            LOGGER.info(f"Will now download some reads for testing - this may take a little while - it might be coffee time.")
            self._download_reads_from_github()
            self._make_test_input(path = self.read_path)
            
        cmd = f"bohra run -i isolates.tab -r {self.reference} -p full --proceed"
        proc = self._run_subprocess(cmd=cmd)

        if proc.returncode == 0:
            LOGGER.info(f"bohra test has completed successfully!!")
        else:
            LOGGER.critical(f"bohra run was not successful... please raise an issue on github.")




#!/usr/bin/env python3
import pathlib, subprocess, argparse, datetime, pandas, re, numpy, jinja2, json, csv
import altair as alt
from Bio import SeqIO

alt.data_transformers.disable_max_rows()

def _get_tree_string(pipeline, wd,phylo):
    '''
    Generate a tree image from a newick
    input:
        :reportdir: the directory where report files are stored
    output:
        string reporesentation of the path to the tree image
    '''
    tree_file = pathlib.Path(wd, 'report','preview.newick') if pipeline == 'preview' else pathlib.Path(wd, 'report','core.newick')
    if tree_file.exists() and phylo == 'true':
        with open(f"{tree_file}", 'r') as t:
            tree = t.read().strip()
    else:
        tree = 'No tree available'
    # print(tree)
    return tree


def _get_offset(reference):
    # print(reference)
    d = {}
    offset = 0
    records = list(SeqIO.parse(reference, "genbank"))
    if records == []:
        records = list(SeqIO.parse(reference, "fasta"))
    if records != []:
        for record in records:
            d[record.id.split('.')[0]] = {'offset' : offset, 'length': len(record.seq)}
            offset += len(record.seq)
    # print(d)
    return d, offset

def get_bin_size(_dict):

    sum_len = 0
    for d in _dict:
        sum_len = sum_len + _dict[d]['length']
    
    _maxbins = int(sum_len/3000)
    if _maxbins == 0:
        print(f"Something has gone wrong - the maxbins value should be > 0.")
    return _maxbins

def check_masked(mask_file, df,wd, _dict ):

    masked = []
    if mask_file != '' and pathlib.Path(wd,mask_file).exists():
        print('blocking out masked regions')
        mask = pandas.read_csv(f"{pathlib.Path(wd,mask_file)}", sep = '\t', header = None, names = ['CHR','Pos1','Pos2'])
        mask['CHR'] = mask['CHR'].astype(str)
        for row in mask.iterrows():
            # print(row[1])
            off = _dict[row[1]['CHR']]['offset']
            l = list(range(row[1]['Pos1'] + off, row[1]['Pos2']+off +1))
            masked.extend(l)

    df['mask'] = numpy.where(df['index'].isin(masked), 'masked', 'unmasked')
    
    return df

def get_contig_breaks(_dict):
    for_contigs = []
    for chromosome in _dict:
        if _dict[chromosome]['length'] > 5000:
            for_contigs.append(_dict[chromosome]['length'] + _dict[chromosome]['offset'])

    return for_contigs


def _plot_snpdensity(reference,wd, isos, mask_file = ''):

    '''
    generate a snp-density accross the genome plot - using the core.tab file
    input:
        :reportdir: the directory where files are kept
    out:
        :snpdensityscript: the javascript string for insert into html
        :snpdenstiydiv: the html div for inster in html doc
    '''
    # helper functions for getting the data into the right format.
    
    _dict,offset = _get_offset(reference = f"{pathlib.Path(wd,reference)}")
    chromosomes = list(_dict.keys())
    
    _maxbins = get_bin_size(_dict = _dict)
    # collate all snps in snps.tab
    vars = {}
    for i in isos:
        # open snps.tab
        snps = pathlib.Path(wd, i, 'snps.tab')
        if snps.exists():
            tab_file = pandas.read_csv(f"{snps}", dtype = str,sep = '\t')
            for chromosome  in chromosomes:
            # print(var)
            # vars = []
                if chromosome not in vars:
                    vars[chromosome] = {}
                chr = tab_file[tab_file['CHROM'] == chromosome]
                # print(chr)
                if not chr.empty:
                    for row in chr.iterrows():
                        # print(row[1]['POS'])
                        pos = int(row[1]['POS'])
                        if pos not in vars[chromosome]:
                            vars[chromosome][pos] = 1
                        else:
                            vars[chromosome][pos] = vars[chromosome][pos] + 1
    # now generate list for x and y value in graph
    data = {}
    for var in vars:
        for pos in vars[var]:
            offset = _dict[var]['offset']
            data[pos + offset] = vars[var][pos]
    df = pandas.DataFrame.from_dict(data, orient='index',columns=['vars']).reset_index()
    # check if mask file used - if yes grey it out in the graph.
    df = check_masked(mask_file = mask_file, df = df, wd = wd, _dict = _dict)
    # get positions of the contig breaks
    for_contigs = get_contig_breaks(_dict = _dict)
    # set colours
    domain = ['masked', 'unmasked']
    range_ = ['#d9dcde', '#216cb8']
    # do bar graphs
    print(mask_file)
    print(df[df['mask'] == 'masked'])
    # if mask_file != 'no_mask':
    bar = alt.Chart(df).mark_bar().encode(
        x=alt.X('index:Q', bin=alt.Bin(maxbins=_maxbins), title = "Core genome position.", axis=alt.Axis(ticks=False)),
        y=alt.Y('sum(vars):Q',title = "Variants observed (per 500 bp)"),
        color=alt.Color('mask', scale = alt.Scale(domain=domain, range=range_), legend=None)
    )

    # generate list of graphs for addition of vertical lines
    graphs = [bar]
    if for_contigs != []:
        for line in for_contigs:
            graphs.append(alt.Chart().mark_rule(strokeDash=[3, 3], size=1, color = 'grey').encode(x = alt.datum(line)))
        
    chart = alt.layer(*graphs).configure_axis(
                    grid=False
                    ).properties(
                        width = 1200
                    ).interactive()

    chart = chart.to_json()
    
    return chart

def _plot_distances(wd):

    '''
    generate a snp-density plot - using the distacnes.tab file
    input:
        :reportdir: the directory where files are kept
    out:
        :distancescript: the javascript string for insert into html
        :distancesdiv: the html div for inster in html doc
    '''
    distance = f"{pathlib.Path(wd, 'report','distances.tab')}"
    try:
        df = pandas.read_csv(distance, sep = '\t')
        # get a list of isolate names
        names = list(df.columns[1:len(df.columns)])
        col1 = df.columns[0]
        
        # if the there is no snp in the isolate (ie same as ref then mak na - then easy to drop)
        
        # collect positions to get allow for histogram and dropna (no snp)
        melted_df = pandas.melt(df, id_vars=[col1], value_vars=names)
        melted_df = melted_df[melted_df[col1]!= melted_df['variable']]
        chart = alt.Chart(melted_df).mark_bar().encode(
                            alt.X('value', axis = alt.Axis(title = 'Pairwise SNP distance')),
                            y=alt.Y('count()', axis= alt.Axis(title = "Frequency"))
                        ).properties(
                                width=1200,
                                height=200
                            )
        chart = chart.to_json()
        return chart
    except:
        return {}

def _subset_versions( _typers, wd, _types ):

    if _types != set():
        vs = pandas.read_csv(f"{pathlib.Path(wd, 'report', 'versions.txt')}", sep = '\t')
        print(vs)
        to_remove = [i for i in vs['tool'].unique() if (i in _typers['typers'] and (i not in _types))]
        vs = vs[~vs['tool'].isin(to_remove)]
        print(vs)
        vs.to_csv(f"{pathlib.Path(wd, 'report', 'versions.txt')}", sep = '\t', index = False)

def _extract_typer( _typers, wd ):
    
    typers = sorted(pathlib.Path(wd).glob(f"*/typer*.txt"))
    print(typers)
    _types = set()
    if typers != []:
        for typer in typers:
            t = typer.name.split('_')[-1].split('.')[0]
            _types.add(t)
    
    print(_types)
    _subset_versions(_typers = _typers, wd = wd, _types = _types)

    return _types


def _get_pan_genome(d, wd):

    image = d[1]['image']
    path = pathlib.Path(wd, 'report', image)
    # print(path)
    if path.exists():
        with open(f"{path}", 'r') as f:
            return f.read().strip()
    else:
        return ''

def _get_isos(wd, iso_list):
    p = pathlib.Path(wd, iso_list)
    with open(p, 'r') as f:
        isos = f.read().strip().split('\n')
    
    return isos

def _generate_table(d, columns,comment, tables, wd, iso_dict, id_col):
    
    if id_col == '':
        return tables,columns,comment
    cols = []
    try:
        with open(f"{pathlib.Path(wd, 'report', d['file'])}", 'r') as f:
            reader = csv.DictReader(f, delimiter = '\t')
            comment[d['link']] = d['comment']
            tables[d['link']] = {'table':[], 'name': d['title'], 'link':d['link']}
            c = 1
            for row in reader:
                if len(row) >1:
                    if row[id_col] in iso_dict:
                        _sample_dict = {'id':iso_dict[row[id_col]]}
                        # if d['link'] == 'snp-distances':
                        #     # print(_sample_dict)
                        for col in row:
                            cols.append(col)
                            _sample_dict[col] = row[col]
                        # if d['link'] == 'snp-distances':
                        #     # print(_sample_dict)
                        tables[d['link']]['table'].append(_sample_dict)
                    else:
                        _data_dict = {'id':c}
                        c = c + 1
                        for col in row:
                            cols.append(col)
                            _data_dict[col] = row[col]
                        tables[d['link']]['table'].append(_data_dict)
    except FileNotFoundError:
        print(f"No file found for {d['title']}")

                        

    cols = list(set(cols))
    if d['columns'] != []:
        columns[d['link']] = d['columns']
    elif 'mlst' in d['link']:
        _c = [{'title':'Isolate','field':'Isolate',"headerFilter":"input","headerFilterPlaceholder":"Search isolate"},{'title':'Scheme','field':'Scheme',"headerFilter":"input","headerFilterPlaceholder":"Search scheme"},{'title':'ST','field':'ST',"headerFilter":"input","headerFilterPlaceholder":"Search ST"}]
        _cls = sorted([c for c in cols if c not in ['Isolate','ST','Scheme']])
        
        if _cls != []:
            for i in _cls:
                _c.append({'title':f"{i}",'field':f"{i}","headerFilter":"input","headerFilterPlaceholder":"Search allele"})
        columns[d['link']] = _c
    
    elif 'snp-distances' in d['link']:
        _c = [{'title':'Isolate','field':'Isolate',"headerFilter":"input","headerFilterPlaceholder":"Search isolate"}]
        _cls = sorted([c for c in cols if c not in ['Isolate']])
        for i in _cls:
            _c.append({'title':f"{i}",'field':f"{i}", "headerVertical":"true", "vertAlign":"bottom"})
        columns[d['link']] = _c

    else:
        _c = [{'title':'Isolate','field':'Isolate',"headerFilter":"input","headerFilterPlaceholder":"Search isolate"}]
        _cls = sorted([c for c in cols if c not in ['Isolate']])
        for i in _cls:
            _c.append({'title':f"{i}",'field':f"{i}","headerFilter":"input","headerFilterPlaceholder":f"Search {i}"})
        columns[d['link']] = _c
    
    return tables,columns,comment

def _get_tables(_data, wd, isos):

    # {
    # link: [
    #       {
                # id: some_int, column_1: data, column_2:data
    #        },
    # ]}
    iso_dict = dict(zip(isos,range(1,len(isos)+1))) # for id in table
    
    tables = {}
    columns = {}
    comment = {}
    
    for d in _data:
        # print(d)
        if d['link'] == 'pan-genome':
            id_col = 'Genes'
        elif d['link'] == 'software-versions':
            id_col = 'tool'
        elif d['link'] == 'pipeline-details':
            id_col = 'detail'
        elif d['type'] == 'table' or d['type'] == 'matrix':
            id_col = 'Isolate'
        else:
            id_col = ''
        # print(id_col)
        tables,columns,comment = _generate_table(d = d, wd = wd, iso_dict= iso_dict, columns= columns,tables=tables,comment=comment,id_col=id_col)
    
    # print(columns["snp-distances"])
    
    return tables,columns,comment
    # pass



def _compile(args):
    
    # get analysis dict
    _dict = json.load(open(f"{pathlib.Path(args.template_dir, 'report_analysis.json')}", 'r'))
    _typers = json.load(open(f"{pathlib.Path(args.template_dir, 'typers.json')}", "r"))
    print(_typers)
    _extract_typer(_typers = _typers, wd = args.launchdir)
    # print(_dict[args.pipeline])
    # for d in _dict[args.pipeline]:
        # print(d)
    
    isos = _get_isos(wd = args.launchdir, iso_list=args.isolates)
    # print(isos)
    reporthtml = pathlib.Path(f'report_{args.pipeline}.html')
    # # path to html template
    indexhtml = pathlib.Path(args.template_dir,'index.html') 
    tables,columns,comment = _get_tables(_data = _dict[args.pipeline], wd = args.launchdir, isos = isos)
    
    data = {
        'newick' :'',
        'job_id':args.job_id[0],
        'pipeline':args.pipeline,
        'date':args.day,
        'user':args.user,
        # 'reference_string':_get_reference_string(reference = args.reference, mask = args.mask, pipeline = args.pipeline),
        'tables':tables,
        'columns': columns,
        'comment':comment,
        'phylo': 'phylo' if args.iqtree == 'true' else 'no_phylo',
        'num_isos':len(isos),
        # 'version_head': version_head,
        # 'version_body':version_body,
        'snp_distances': _plot_distances(wd = args.launchdir) if args.pipeline not in ['preview', 'assemble','amr_typing'] else {0:0},
        'snp_density': _plot_snpdensity(reference=args.reference, wd = args.launchdir, isos = isos, mask_file = args.mask) if args.pipeline not in ['preview', 'assemble','amr_typing'] else {0:0},
        'pan_svg': _get_pan_genome(d = _dict[args.pipeline], wd = args.launchdir) if args.pipeline == 'full' else ''
        }
    
    data['newick'] = _get_tree_string(pipeline = args.pipeline, wd = args.launchdir, phylo = args.iqtree)
    
    print("rendering html")
    report_template = jinja2.Template(pathlib.Path(indexhtml).read_text())
    reporthtml.write_text(report_template.render(data))
    

def set_parsers():
    # setup the parser
    parser = argparse.ArgumentParser(description='Collate and write bohra report',formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('--pipeline',
        help='',
        default = 'default')
    parser.add_argument('--launchdir',
        help='',
        default = '')
    parser.add_argument('--contigs',
        help='',
        default = '')
    parser.add_argument('--mask',
        help='',
        default = '')
    parser.add_argument('--template_dir',
        help='',
        default = '')
    parser.add_argument('--day',
        help=f'',
        default = '')
    parser.add_argument('--user', 
        help='',
        default = '')
    parser.add_argument('--job_id',
        help='',
        default = '',
        nargs='+')
    parser.add_argument('--isolates',
        help = '',
        default = '')
    parser.add_argument('--reference',
        help = '',
        default = '')
    parser.add_argument('--iqtree',
        help = '',
        default = '')
    
    
    
    
    parser.set_defaults(func=_compile)
    args = parser.parse_args()
    
    if vars(args) == {}:
        parser.print_help(sys.stderr)
    else:
        # print(args)
        args.func(args)
	

def main():
    """
    run pipeline
    """

    args = set_parsers()
    

if __name__ == "__main__":
    main()




#!/usr/bin/env python3
import pathlib, pandas, math, sys,  re


def get_genes(_file, _type, _dict):
    
    with open(_file, 'r') as f:
        data = f.read().strip().split('\n')
        header = data[0].split('\t')
        line = data[1].split('\t')
        for i in range(1,len(header)):
            print(header[i])
            l = [gn for gn in line[i].split(',')] if _type != 'partials' else [f"{gn}^" for gn in line[i].split(',')]
            # l = line[i] if _type != 'partials' else f"{line[i]}^"
            # print(line[i])
            print(l)
            if header[i] in _dict:
                    _dict[header[i]].extend(l)
            else:
                _dict[header[i]] = l
        return _dict

_dict = {'Isolate': sys.argv[1]}

# matches
_dict = get_genes(_file = sys.argv[2], _type = 'matches', _dict = _dict)

# partials 
_dict = get_genes(_file = sys.argv[3], _type = 'partials', _dict = _dict)

for d in _dict:
    if d != 'Isolate':
        _dict[d] = ';'.join(_dict[d])
df = pandas.DataFrame(_dict, index = [0])
df = df.set_index('Isolate')
print(df)
df.to_csv('resistome.txt', sep = '\t')



import sys, pathlib, pandas, pytest, numpy,logging, subprocess,os, json

from unittest.mock import patch

from bohra.SnpDetection import RunSnpDetection
# from bohra.bohra.ReRunSnpDetection import ReRunSnpDetection

script_path = pathlib.Path(__file__).parent.parent / 'tests'
conda_path = pathlib.Path(f"{os.environ['CONDA_PREFIX']}").parent
outdir = pathlib.Path.cwd()

with open(f"{script_path / 'variables.json'}", 'r') as j:
        cfg = json.load(j)


def get_main(tool):

        return cfg[f"{tool}"]["main"]

def get_input(tool):
        return cfg[f"{tool}"]["input"]

def get_truth(tool):
        return cfg[f"{tool}"]["truth"]

def get_paths(tool):
        nf = script_path / get_main(tool)
        cp= script_path / get_input(tool)
        tr = script_path / get_truth(tool)
        return nf,cp,tr
def get_reference(tool):
        return script_path / f"{cfg[tool]['ref']}"

def clean():

        subprocess.run(f"rm -rf .nextflow* bohra.log work for_testing core.* clean.*", shell = True)
   


@pytest.mark.nf_snps
def test_nf_snippy_clean():
        nf,cp,tr = get_paths("snippy_clean")
        cmd = f"nextflow run {nf} --aln_path {cp}  --outdir {outdir} --publish_dir_mode copy --enable_conda true -with-conda --truth {tr}"
        proc = subprocess.run(cmd, shell=True, capture_output=True)
        
        assert proc.returncode == 0
        clean()

@pytest.mark.nf_snps
def test_nf_snp_dists():
        nf,cp,tr = get_paths("snp_dists")
        cmd = f"nextflow run {nf} --aln_path {cp} --outdir {outdir} --publish_dir_mode copy --enable_conda true -with-conda --truth {tr}"
        proc = subprocess.run(cmd, shell=True, capture_output=True)
        
        assert proc.returncode == 0
        clean()

@pytest.mark.nf_snps
def test_nf_snippy_core():
        nf,cp,tr = get_paths("snippy_core")
        ref = get_reference(tool = 'snippy')
        cmd = f"nextflow run {nf} --aln_path {cp} --outdir {outdir} --reference {ref} --mask_string no_mask --publish_dir_mode copy --enable_conda true -with-conda --truth {tr}"
        proc = subprocess.run(cmd, shell=True, capture_output=True)
        
        assert proc.returncode == 0
        clean()

@pytest.mark.nf_kraken
def test_nf_kraken():
        nf,cp,tr = get_paths("kraken2")
        ref = get_reference(tool = 'kraken2')
        cmd = f"nextflow run {nf} --read_path {cp}  --kraken2_db {ref} --publish_dir_mode copy --profile mdu --enable_conda true -with-conda "
        proc = subprocess.run(cmd, shell=True, capture_output=True)
        
        assert proc.returncode == 0
        clean()

@pytest.mark.nf_snps
def test_nf_snippy():
        nf,cp,tr = get_paths("snippy")
        ref = get_reference()
        cmd = f"nextflow run {nf} --read_path {cp}  --outdir {outdir} --reference {ref} --publish_dir_mode copy --enable_conda true -with-conda --truth {tr} --minmap 60 --basequal 13 --minqual 100 --minfrac 0 --mincov 10"
        proc = subprocess.run(cmd, shell=True, capture_output=True)
        
        assert proc.returncode == 0
        clean()


@pytest.mark.nf_typing
def test_nf_ngmaster():
        nf,cp,tr = get_paths("ngmaster")
        
        cmd = f"nextflow run {nf} --contig_path {cp} --outdir {outdir} --publish_dir_mode copy --enable_conda true -with-conda --truth {tr}"
        proc = subprocess.run(cmd, shell=True, capture_output=True)
        
        assert proc.returncode == 0
        clean()

@pytest.mark.nf_typing
def test_nf_meningotype():
        nf,cp,tr = get_paths("meningotype")
        
        cmd = f"nextflow run {nf} --contig_path {cp}  --outdir {outdir} --publish_dir_mode copy --enable_conda true -with-conda --truth {tr}"
        proc = subprocess.run(cmd, shell=True, capture_output=True)
        
        assert proc.returncode == 0
        clean()

@pytest.mark.nf_typing
def test_nf_lissero():
        nf,cp,tr = get_paths("lissero")
        
        cmd = f"nextflow run {nf} --contig_path {cp}  --outdir {outdir} --publish_dir_mode copy --enable_conda true -with-conda --truth {tr}"
        proc = subprocess.run(cmd, shell=True, capture_output=True)
        
        assert proc.returncode == 0
        clean()

@pytest.mark.nf_typing
def test_nf_kleborate():
        nf,cp,tr = get_paths("kleborate")
        
        cmd = f"nextflow run {nf} --contig_path {cp}  --outdir {outdir} --publish_dir_mode copy --enable_conda true -with-conda --truth {tr}"
        proc = subprocess.run(cmd, shell=True, capture_output=True)
        
        assert proc.returncode == 0
        clean()

@pytest.mark.nf_typing
def test_nf_emmtyper():
        nf,cp,tr = get_paths("emmtyper")
        
        cmd = f"nextflow run {nf} --contig_path {cp} --outdir {outdir} --publish_dir_mode copy --enable_conda true -with-conda --truth {tr}"
        proc = subprocess.run(cmd, shell=True, capture_output=True)
        
        assert proc.returncode == 0
        clean()

@pytest.mark.nf_typing
def test_nf_ectyper():
        nf,cp,tr = get_paths("ectyper")
        
        cmd = f"nextflow run {nf} --contig_path {cp} --outdir {outdir} --publish_dir_mode copy --enable_conda true -with-conda --truth {tr}"
        proc = subprocess.run(cmd, shell=True, capture_output=True)
        
        assert proc.returncode == 0
        clean()

@pytest.mark.nf_typing
def test_nf_mlst():
        nf,cp,tr = get_paths("mlst")
        
        cmd = f"nextflow run {nf} --contig_path {cp}  --outdir {outdir} --publish_dir_mode copy --enable_conda true -with-conda --blast_db no_db --data_dir no_db --mlst_exclude 'ecoli,abaumannii' --truth {tr}"
        proc = subprocess.run(cmd, shell=True, capture_output=True)
        
        assert proc.returncode == 0
        clean()

@pytest.mark.setup_bohra
def test_name_string():
        '''
        assert true when the input is a string > len 0
        '''
        with patch.object(RunSnpDetection, "__init__", lambda x: None):
                detect_obj = RunSnpDetection()
                detect_obj.logger = logging.getLogger(__name__)
                assert detect_obj._name_exists('somestring')

@pytest.mark.setup_bohra
def test_name_non_string():
        '''
        if a non string input is used return false
        '''
        with patch.object(RunSnpDetection, "__init__", lambda x: None):
                detect_obj = RunSnpDetection()
                detect_obj.logger = logging.getLogger(__name__)
                with pytest.raises(SystemExit):
                        detect_obj._name_exists(9)

@pytest.mark.setup_bohra
def test_name_empty_string():
        '''
        confirm that False is returned if a 0 length strin is input
        '''
        with patch.object(RunSnpDetection, "__init__", lambda x: None):
                detect_obj = RunSnpDetection()
                detect_obj.logger = logging.getLogger(__name__)
                with pytest.raises(SystemExit):
                        detect_obj._name_exists('')

@pytest.mark.setup_bohra            
def test_path_exists():

        '''
        confirm true is returned when path is found
        '''
        with patch.object(RunSnpDetection, "__init__", lambda x: None):
                detect_obj = RunSnpDetection()
                detect_obj.logger = logging.getLogger(__name__)
                some_path = pathlib.Path(__file__).parent
                assert detect_obj._path_exists(some_path)

@pytest.mark.setup_bohra           
def test_not_path_exists():

        '''
        confirm true is returned when path is found
        '''
        with patch.object(RunSnpDetection, "__init__", lambda x: None):
                detect_obj = RunSnpDetection()
                detect_obj.logger = logging.getLogger(__name__)
                some_path = pathlib.Path('some_file.txt')
                assert not detect_obj._path_exists(some_path)



#!/usr/bin/env python3
import sys, json, datetime, subprocess, pathlib,pandas


def get_vals_seqtk(f):

    with open(f, 'r') as inputfile:
        lines = inputfile.read().split('\n')
        qscore = lines[1].split('\t')[3]
        bases = lines[1].split('\t')[0]
        gc= float(lines[1].split('\t')[1]) + float(lines[1].split('\t')[2])
        return float(qscore), int(bases), round(gc,1)

def sum_stats(f,cmd,col):

    p = subprocess.run(f"cat {f} | datamash {cmd} --header-in {col}", shell = True, capture_output = True, encoding= 'utf-8')

    return p.stdout

def get_vals_kit(f):

    reads = sum_stats(f, 'sum', 4)
    minval = sum_stats(f, 'mean', 6)
    avgval = sum_stats(f, 'mean', 7)
    maxval = sum_stats(f, 'mean', 8)

    return int(reads.strip()), float(minval.strip()), float(avgval.strip()), float(maxval.strip())

def get_length(genome_size):

    with open(genome_size, 'r') as f:
        x = f.read().strip().split('\n')[0].strip()
        try:
            return int(x)
        except:
            print(f"Something has gone horribly wrong {x} can not be parsed to an int!!")
            raise SystemExit
    
def get_dpth(genome_size,bases):
    length = get_length(genome_size)
    dpth = int(bases)/length
    
    return round(dpth, 1)

gcs = pandas.read_csv(sys.argv[3], sep = '\t')
tab = pandas.read_csv(sys.argv[2], sep = '\t')
tab['Isolate'] = sys.argv[1]
tab = tab.rename(columns = {'num_seqs': 'Reads', 'sum_len': 'Yield','min_len':'Min len','max_len':'Max len', 'avg_len':'Avg len', 'Q30(%)': 'Average quality (% >Q30)'})
tab['Estimated average depth'] = get_dpth(genome_size = sys.argv[4], bases = tab['Yield'].values[0])
tab['GC content'] = gcs[gcs.columns[0]].values[0]
tab['Avgerage quality'] = gcs[gcs.columns[1]].values[0]
tab = tab[['Isolate','Reads','Yield','GC content','Min len','Avg len','Max len','Average quality (% >Q30)','Estimated average depth']]
tab.to_csv('read_assessment.txt', sep = '\t', index = False)





    





#!/usr/bin/env python3
import pathlib, subprocess, sys, pandas, numpy, csv

import pandas, pathlib


_file = sys.argv[1]

results = {'Isolate': sys.argv[2]}
with open(_file, 'r') as f:
    reader = csv.DictReader(f, delimiter = '\t')
    for row in reader:
        if row['mash_nearest_neighbor'] not in results:
            results[row['mash_nearest_neighbor']] = f"Contigs: {row['num_contigs']} - total length {row['size']}; Mash_distance: {row['mash_neighbor_distance']}"
            

header = []

df = pandas.DataFrame(results, index = [0])
df.to_csv('plasmid.txt', sep = '\t', index = False)



import pathlib
import os,getpass
import pandas
import jinja2
import sh
import logging
import filecmp
import datetime
import numpy
import itertools
import subprocess
import re
import toml
import shutil
from packaging import version
from Bio import SeqIO, Phylo
from bohra.SnpDetection import RunSnpDetection
# from bohra.bohra_logger import logger
class UpdateBohra(RunSnpDetection):
    '''
    A class to update older bohra directories directories to new bohra structure
    '''
    def __init__(self,args):
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.INFO)
        # create file handler which logs even debug messages
        fh = logging.FileHandler('bohra.log')
        fh.setLevel(logging.INFO)
        # create console handler with a higher log level
        ch = logging.StreamHandler()
        ch.setLevel(logging.INFO)
        # create formatter and add it to the handlers
        formatter = logging.Formatter('[%(levelname)s:%(asctime)s] %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')
        fh.setFormatter(formatter)
        ch.setFormatter(formatter)
        # add the handlers to the logger
        self.logger.addHandler(ch)
        self.logger.addHandler(fh)

        # get date and time
        self.now = datetime.datetime.today().strftime("%d_%m_%y_%H")
        self.day = datetime.datetime.today().strftime("%d_%m_%y")
        # get the working directory
        self.workdir = pathlib.Path(args.workdir)
        self.job_id = self._name_exists(args.job_id)
        self.reference = pathlib.Path(args.reference) if args.reference != '' else ''
        if args.input_file == '':
                self.logger.warning('Input file can not be empty, please set -i path_to_input to try again')
                raise SystemExit()
        else:
            self.input_file = pathlib.Path(args.input_file)
    
    def _update_timestamps(self):
        '''
        if snippy files or assembly present update timestamp so they will not be rerun after linking reads to directory
        '''
        self.logger.info(f"Updating timestamps to prevent uneccesary reruns of snippy and assembler. If you would like to force a rerun use the -F option.")
        for i in self.isolates:
            p = self.jobdir / i
            for f in p.iterdir():
                if "snps" in f"{f}" or f"{f}".endswith(".fa"):
                    cmd = f"touch -m {f}"
                    subprocess.run(cmd, shell = True, capture_output=True)


    def rm_old_reads(self):
        '''
        if READS directory present remove it
        '''

        rds = self.jobdir / 'READS'
        if rds.exists():
            subprocess.run(f"rm -r {rds}", shell = True, capture_output= True)
            # rds.rmdir()
    def archive_old_report(self):
        '''
        if report directory present rename it
        '''

        r = self.jobdir / 'report'
        if r.exists():
            subprocess.run(f"mv {r} {r}_archive", shell = True, capture_output= True)
            # rds.rmdir()
    def _find_reads(self):
        '''
        a function to find sample directories
        '''
        tab = pandas.read_csv(self.input_file, sep = None, engine = 'python', header = None)

        self.jobdir = self.workdir / self.job_id
        self.logger.info(f"Looking for {self.jobdir}")
        if self.jobdir.exists():
            self.logger.info(f"{self.jobdir} found!")
            self.rm_old_reads()
            self.check_input_structure(tab = tab)
            self.check_reads_exists(tab = tab)
            self.isolates = [i.strip('#') for i in tab.iloc[ : , 0]]
            self.archive_old_report()
        else:
            self.logger.warning(f"{self.job_id} does not exist. Are you sure you want to reconfigure this job? Please check help and try again.")
        # pass
    
    def make_tomls(self):
        self.logger.info(f"Checking if tomls need to be made for SNP detection and/or assemblies. This should avoid rerunning these.")
        for i in self.isolates:
            snps = self.jobdir / i / 'snps.aligned.fa'
            if snps.exists():
                d = {i:
                {'snippy':
                    {
                    'reference': f"{self.reference}",
                    'run_snippy': 'Yes',
                    'alignment': f"{i}/snps.aligned.fa",
                    'vcf': f"{i}/snps.vcf",
                    'cmd': f"snippy --outdir {i} --ref {self.reference} --R1 {i}/R1.fq.gz --R2 {i}/R2.fq.gz --force --cpus 8",
                    }
                }
            }
                with open(f"{ self.jobdir / i / 'snippy.toml'}", 'wt') as f:
                    toml.dump(d, f)
            contigs = self.jobdir / i / 'contigs.fa'
            if contigs.exists():
                c = {i:
                {
                    'assembly':
                    {
                        'done': 'Yes'
                    }
                }}
                with open(f"{ self.jobdir / i / 'assembly.toml'}", 'wt') as f:
                    toml.dump(c, f)


    def update(self):

        self._find_reads()
        self._update_timestamps()
        self.make_tomls()


class Nulla2bohra(UpdateBohra):

    '''
    convert a nullarbor directory to a bohra directory
    '''

    def __init__(self,args):
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.INFO)
        # create file handler which logs even debug messages
        fh = logging.FileHandler('bohra.log')
        fh.setLevel(logging.INFO)
        # create console handler with a higher log level
        ch = logging.StreamHandler()
        ch.setLevel(logging.INFO)
        # create formatter and add it to the handlers
        formatter = logging.Formatter('[%(levelname)s:%(asctime)s] %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')
        fh.setFormatter(formatter)
        ch.setFormatter(formatter)
        # add the handlers to the logger
        self.logger.addHandler(ch)
        self.logger.addHandler(fh)

        # get date and time
        self.now = datetime.datetime.today().strftime("%d_%m_%y_%H")
        self.day = datetime.datetime.today().strftime("%d_%m_%y")
        # get the working directory
        self.reference = pathlib.Path(args.reference) if args.reference != '' else ''
        self.workdir = pathlib.Path(args.workdir)
        self.job_id = self._name_exists(args.job_id)
        if args.input_file == '':
                self.logger.info('Checking your job folder for an input file')
                self.input_file = self.check_for_input_file()
        else:
            self.input_file = pathlib.Path(args.input_file)

    def check_for_input_file(self):

        p = self.workdir / self.job_id

        infile = p / 'input.tab'
        if infile.exists():
            return f"{infile}"
        else:
            self.logger.warning(f"You have not supplied an input file and none can be found in your {self.job_id} directory. Please supply an input file and try again.")
            raise SystemExit

        
class CheckDeps(RunSnpDetection):

    def __init__(self, args):
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.INFO)
        # create file handler which logs even debug messages
        fh = logging.FileHandler('bohra.log')
        fh.setLevel(logging.INFO)
        # create console handler with a higher log level
        ch = logging.StreamHandler()
        ch.setLevel(logging.INFO)
        # create formatter and add it to the handlers
        formatter = logging.Formatter('[%(levelname)s:%(asctime)s] %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')
        fh.setFormatter(formatter)
        ch.setFormatter(formatter)
        # add the handlers to the logger
        self.logger.addHandler(ch)
        self.logger.addHandler(fh)
        self.kraken_db = args.kraken_db 

    def check_sa(self):
        deps = ['shovill', 'skesa', 'spades.py']
        self.check_quals()
        self.check_snippycore()
        self.check_snpdists()
        self.check_kraken2DB()
        # self.check_iqtree()
        # self.check_assembler()
        self.check_assemble_accesories()
        for d in deps:
            self.check_installation(software = d)


    def check(self):
        """
        for bohra check
        """

        self.check_sa()
        self.check_roary()
        # self.logger.info("Well done, you have all dependencies installed correctly!!")
            


"""Bohra 
.. moduleauthor:: Kristy Horan <kristyhoran15@gmail.com>

Bohra exinct tree kangaroo that lived on the nullarbor

Bohra is microbial genomics pipeline, designed predominantly for use in public health, but may also be useful in research settings. The pipeline takes as input a tab-delimited file with the isolate IDs followed by the path to READ1 and READ2, a reference for alignment and a unique identifier, where reads are illumina paired end reads (other platforms are not supported).

The pipline is based on nullarbor (https://github.com/tseemann/nullarbor) and is designed to be run in high performance computing environment.

Bohra is modular allowing the user to choose between calling SNPs and generating a phylogenetic tree, perform assemblies and detect AMR, perform typing etc; Or use the full pipeline to call SNPs, generate phylogenies, assemble, type and detect the pan-geneome. The output of Bohra is a html report that can be distributed, with downloadable tables and data.
"""


# import logging
import argparse
import pathlib
import sys
import os
import shutil
from bohra.SnpDetection import RunSnpDetection, SetupInputFiles, InitBohra, TestBohra
from bohra.version import version


def run_pipeline(args):
    '''
    Run the pipeline for the first time
    '''
    
    R = RunSnpDetection(args)
    return(R.run_pipeline())

def find_reads(args):

    S = SetupInputFiles(args)
    S.find_reads()

def init_bohra():
    
    I = InitBohra()
    I.init()

def test_bohra(args):
    
    S = TestBohra(args)
    S.run_tests()
    # run_pipeline(inputs)

def main():
    # setup the parser
    # Check if CONDA_PREFIX is set
    conda_var = os.getenv('CONDA_PREFIX')
    if conda_var is None:
        # Set the default value
        conda_prefix = 'NO CONDA INSTALLED'
        # Set the default value
    else:
        conda_prefix = f"{pathlib.Path(os.getenv('CONDA_PREFIX')).parent}"
    
    parser = argparse.ArgumentParser(description=f'Bohra - a bacterial genomics pipeline - version {version}',formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + version)
    # options for running
    subparsers = parser.add_subparsers(help="Task to perform")
    # run bohra pipeline
    parser_run = subparsers.add_parser('run', help='Run bohra', formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser_run.add_argument('--check',action="store_true", help = "Check that dependencies are installed correctly.")
    parser_run.add_argument('--input_file','-i',help='Path to reads file, which is a tab-delimited with 3 columns <isolatename>  <path_to_read1> <path_to_read2>. REQUIRED', default='')
    parser_run.add_argument('--contigs','-c',help='Path to contigs file, which is a tab-delimited with 3 columns <isolatename>  <path_to_contigs>. OPTIONAL if you already have assemblies.', default='')
    parser_run.add_argument('--job_id','-j',help='Job ID is the name that will be displayed on your report', default='Bohra microbial genomics pipeline')
    parser_run.add_argument('--reference','-r',help='Path to reference (.gbk or .fa)', default = '')
    parser_run.add_argument('--mask','-m',default = '', help='Path to mask file if used (.bed)')
    parser_run.add_argument("--abritamr_args",default="",help="Set if you would like to use point mutations, please provide a valid species.", choices= ['Neisseria', 'Acinetobacter_baumannii', "Campylobacter", "Enterococcus_faecalis", "Enterococcus_faecium", "Escherichia", "Klebsiella", "Salmonella", "Staphylococcus_aureus", "Staphylococcus_pseudintermedius", "Streptococcus_agalactiae", "Streptococcus_pneumoniae", "Streptococcus_pyogenes", "Vibrio_cholerae"])
    parser_run.add_argument('--kraken_db', '-k', default="KRAKEN2_DEFAULT_DB", help="Path to DB for use with kraken2, if no DB present speciation will not be performed.")
    parser_run.add_argument('--pipeline','-p', default = 'preview', choices=['preview','default','full','snps','phylogeny','assemble','amr_typing'], help=f"The pipeline to run. `preview` - generates a rapid tree using mash distances | `default` - runs snippy, phylogenetic tree (if > 3 sequences), assemblies, mlst and amr gene detection | `all` - same as default but includes roary pangenome analysis")
    parser_run.add_argument('--assembler','-a', default = 'shovill', choices=['shovill','skesa','spades'], help=f"Assembler to use (shovill uses spades > 3.14 with --isolate mode).")
    parser_run.add_argument('--spades_args', default = "", help="Use to add arguments to spades (when running with --assembler spades) for example: '--cov-cutoff auto' ")
    parser_run.add_argument('--cpus',help='Number of max CPU cores to run, will define how many rules are run at a time, if 0 then the avail cpus will be determined at time of launch', default=0) # need to change
    parser_run.add_argument('--minmap','-mp',help='Snippy - minimum read mapping quality to consider.', default='60')
    parser_run.add_argument('--basequal','-bq',help='Snippy - Minimum base quality to consider.', default='13')
    parser_run.add_argument('--minqual','-mq',help='Snippy - minumum QUALITY in VCF column 6', default='100')
    parser_run.add_argument('--minfrac','-mf',help='Snippy - minumum proportion for variant evidence ', default='0')
    parser_run.add_argument('--mincov','-mc',help='Snippy - minimum site depth to for calling alleles.', default='10')
    parser_run.add_argument('--workdir','-w', default = f"{pathlib.Path.cwd().absolute()}", help='The directory where Bohra will be run, default is current directory') # don't need this
    parser_run.add_argument('--force','-f', action="store_true", help = "Add if you would like to force a complete restart of the pipeline. All previous logs will be lost.")
    parser_run.add_argument('--no_phylo',action="store_true", help = "Set if you do NOT want to generate a phylogentic tree.")
    parser_run.add_argument('--config', default = f"", help='An additional config file, required if running on a non-local machine, ie slurm, cloud. For help see documentation at https://github.com/MDU-PHL/bohra or https://www.nextflow.io/docs/latest/executor.html') # don't need this
    parser_run.add_argument('--profile', default = f"", help='The resource profile to use. Defaults to local, if using an alternative config file, this calue should represent the name of a profile provided') 
    parser_run.add_argument('--conda_path', default = conda_prefix, help='The path to where your pre-installed conda envs are stored, defaults to installing conda envs in your work directory. This can be provided in your profiles settings as well - it assumes you have pre-configured all of your conda environments for each process run by bohra, this is an advanced setting. Please take care.') 
    parser_run.add_argument('--blast_db', default = f"{os.getenv('BLAST_DB', '')}", help='Path to the mlst blast_db, defaults to what is installed in the environment.') 
    parser_run.add_argument('--data_dir', default = f"{os.getenv('PUBMLST_DB','')}", help='Path to the mlst datadir, defaults to what is installed in the environment.') 
    parser_run.add_argument('--mlst_exclude',default = f"", help='Space delimited list of mlst schemes to exclude.', nargs='+')
    parser_run.add_argument('--mobsuite_db', default = f"{os.getenv('MOBSUITE_DB','')}", help='Path to the mobsuite_db, defaults to what is installed in the environment.') 
    parser_run.add_argument('--proceed', action="store_true", help = "If you would like to proceed straigt to the pipeline.")
    parser_run.add_argument('--gubbins',  action = 'store_true', help = 'Set to use gubbins for recombination correction.')
    parser_run.add_argument('--keep', default = 'N', choices= ['Y', 'N'], help = 'If you are rerunning bohra over an exisiting directory set --keep to \'Y\' to archive report files - otherwise previous reprot files will be removed.')
    parser_run.add_argument('--no-conda',action="store_true", help="Set if you DO NOT WANT to use separate conda environments for each nextflow process.")

    parser_setup = subparsers.add_parser('generate_input', help='Generate input files for bohra', formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_setup.add_argument(
        '--read_path',
        '-r',
        default='',
        help = 'Path to look for read files')
    parser_setup.add_argument(
        '--isolate_list',
        '-i',
        default= '',
        help = 'List of isolates (one isolate name per line) to include in input file. If not provided all sequences in found will be included.'
    )

    parser_init = subparsers.add_parser('init', help='Setup bohra dependencies', formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_check = subparsers.add_parser('check', help='Check for bohra dependencies', formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser_test = subparsers.add_parser('test', help='Test bohra', formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_test.add_argument(
        '--reference',
        '-r',help='Reference to be used in test.', 
        default='Lm_Cluster1_J1-108.fa'
    )
    
    parser_run.set_defaults(func=run_pipeline)
    parser_setup.set_defaults(func=find_reads)
    parser_test.set_defaults(func=test_bohra)
    

        
    args = parser.parse_args()
    print(len(sys.argv))
    if len(sys.argv) < 2:
        parser.print_help(sys.stderr)
    elif sys.argv[1] in ['init', 'check']:
        init_bohra()
    else:
        args.func(args)
	
if __name__ == '__main__':
    main()

